@@ -162,8 +162,8 @@ int _GMP_miller_rabin(mpz_t n, mpz_t a)
 }
 
 /* Returns Lucas sequence  U_k mod n and V_k mod n  defined by P,Q */
-void _GMP_lucas_seq(mpz_t U, mpz_t V, mpz_t n, IV P, IV Q, mpz_t k,
-                    mpz_t Qk, mpz_t t)
+void lucas_seq(mpz_t U, mpz_t V, mpz_t n, IV P, IV Q, mpz_t k,
+               mpz_t Qk, mpz_t t)
 {
   UV b = mpz_sizeinbase(k, 2);
   IV D = P*P - 4*Q;
@@ -179,6 +179,12 @@ void _GMP_lucas_seq(mpz_t U, mpz_t V, mpz_t n, IV P, IV Q, mpz_t k,
     mpz_set_ui(V, 2);
     return;
   }
+  if (mpz_even_p(n)) {
+    /* If n is even, then we can't divide by 2.  Do it differently. */
+    alt_lucas_seq(U, V, n, P, Q, k, Qk, t);
+    return;
+  }
+
   mpz_set_ui(U, 1);
   mpz_set_si(V, P);
   mpz_set_si(Qk, Q);
@@ -256,6 +262,61 @@ void _GMP_lucas_seq(mpz_t U, mpz_t V, mpz_t n, IV P, IV Q, mpz_t k,
   mpz_mod(V, V, n);
 }
 
+void alt_lucas_seq(mpz_t Uh, mpz_t Vl, mpz_t n, IV P, IV Q, mpz_t k,
+                   mpz_t Ql, mpz_t t)
+{
+  mpz_t Vh, Qh;
+  int j, s = mpz_scan1(k,0), b = mpz_sizeinbase(k,2);
+
+  if (mpz_sgn(k) <= 0) {
+    mpz_set_ui(Uh, 0);
+    mpz_set_ui(Vl, 2);
+    return;
+  }
+
+  mpz_set_ui(Uh, 1);
+  mpz_set_ui(Vl, 2);
+  mpz_set_ui(Ql, 1);
+  mpz_init_set_si(Vh,P);
+  mpz_init_set_ui(Qh,1);
+
+  for (j = b; j > s; j--) {
+    mpz_mul(Ql, Ql, Qh);
+    if (mpz_tstbit(k, j)) {
+      mpz_mul_si(Qh, Ql, Q);
+      mpz_mul(Uh, Uh, Vh);
+      mpz_mul_si(t, Ql, P);  mpz_mul(Vl, Vl, Vh); mpz_sub(Vl, Vl, t);
+      mpz_mul(Vh, Vh, Vh); mpz_sub(Vh, Vh, Qh); mpz_sub(Vh, Vh, Qh);
+    } else {
+      mpz_set(Qh, Ql);
+      mpz_mul(Uh, Uh, Vl);  mpz_sub(Uh, Uh, Ql);
+      mpz_mul_si(t, Ql, P);  mpz_mul(Vh, Vh, Vl); mpz_sub(Vh, Vh, t);
+      mpz_mul(Vl, Vl, Vl);  mpz_sub(Vl, Vl, Ql);  mpz_sub(Vl, Vl, Ql);
+    }
+    mpz_mod(Qh, Qh, n);
+    mpz_mod(Uh, Uh, n);
+    mpz_mod(Vh, Vh, n);
+    mpz_mod(Vl, Vl, n);
+  }
+  mpz_mul(Ql, Ql, Qh);
+  mpz_mul_si(Qh, Ql, Q);
+  mpz_mul(Uh, Uh, Vl);  mpz_sub(Uh, Uh, Ql);
+  mpz_mul_si(t, Ql, P);  mpz_mul(Vl, Vl, Vh);  mpz_sub(Vl, Vl, t);
+  mpz_mul(Ql, Ql, Qh);
+  mpz_clear(Qh);  mpz_clear(Vh);
+  mpz_mod(Ql, Ql, n);
+  mpz_mod(Uh, Uh, n);
+  mpz_mod(Vl, Vl, n);
+  for (j = 0; j < s; j++) {
+    mpz_mul(Uh, Uh, Vl);
+    mpz_mul(Vl, Vl, Vl);  mpz_sub(Vl, Vl, Ql);  mpz_sub(Vl, Vl, Ql);
+    mpz_mul(Ql, Ql, Ql);
+    mpz_mod(Ql, Ql, n);
+    mpz_mod(Uh, Uh, n);
+    mpz_mod(Vl, Vl, n);
+  }
+}
+
 void lucasuv(mpz_t Uh, mpz_t Vl, IV P, IV Q, mpz_t k)
 {
   mpz_t Vh, Ql, Qh, t;
@@ -382,8 +443,8 @@ int lucas_lehmer(UV p)
 /* Returns:  -1 unknown, 0 composite, 2 definitely prime */
 int llr(mpz_t N)
 {
-  mpz_t v, k, V;
-  UV i, n;
+  mpz_t v, k, V, U, Qk, t;
+  UV i, n, P;
   int res = -1;
 
   if (mpz_cmp_ui(N,100) <= 0) return (_GMP_is_prob_prime(N) ? 2 : 0);
@@ -401,18 +462,30 @@ int llr(mpz_t N)
     goto DONE_LLR;
 
   mpz_init(V);
+  mpz_init(U); mpz_init(Qk); mpz_init(t);
   if (!mpz_divisible_ui_p(k, 3)) { /* Select V for 3 not divis k */
-    mpz_t U, Qk, t;
-    mpz_init(U); mpz_init(Qk); mpz_init(t);
-    _GMP_lucas_seq(U, V, N, 4, 1, k, Qk, t);
-    mpz_clear(t);  mpz_clear(Qk); mpz_clear(U);
+    lucas_seq(U, V, N, 4, 1, k, Qk, t);
   } else if ((n % 4 == 0 || n % 4 == 3) && mpz_cmp_ui(k,3)==0) {
     mpz_set_ui(V, 5778);
   } else {
-    /* TODO: No logic for the k | 3 case yet. */
-    mpz_clear(V);
-    goto DONE_LLR;
+    /* Öystein J. Rödseth: http://www.uib.no/People/nmaoy/papers/luc.pdf */
+    for (P=5; P < 1000; P++) {
+      mpz_set_ui(t, P-2);
+      if (mpz_jacobi(t, N) == 1) {
+        mpz_set_ui(t, P+2);
+        if (mpz_jacobi(t, N) == -1) {
+          break;
+        }
+      }
+    }
+    if (P >= 1000) {
+      mpz_clear(t);  mpz_clear(Qk); mpz_clear(U);
+      mpz_clear(V);
+      goto DONE_LLR;
+    }
+    lucas_seq(U, V, N, P, 1, k, Qk, t);
   }
+  mpz_clear(t);  mpz_clear(Qk); mpz_clear(U);
 
   for (i = 3; i <= n; i++) {
     mpz_mul(V, V, V);
@@ -427,6 +500,41 @@ DONE_LLR:
   mpz_clear(k); mpz_clear(v);
   return res;
 }
+/* Returns:  -1 unknown, 0 composite, 2 definitely prime */
+int proth(mpz_t N)
+{
+  mpz_t v, k, a;
+  UV n;
+  int i, res = -1;
+  if (mpz_cmp_ui(N,100) <= 0) return (_GMP_is_prob_prime(N) ? 2 : 0);
+  if (mpz_even_p(N) || mpz_divisible_ui_p(N, 3)) return 0;
+  mpz_init(v); mpz_init(k);
+  mpz_sub_ui(v, N, 1);
+  n = mpz_scan1(v, 0);
+  mpz_tdiv_q_2exp(k, v, n);
+  /* N = k * 2^n + 1 */
+  if (mpz_sizeinbase(k,2) <= n) {
+    const unsigned char sprimes[25] = {2,3,5,7,11,13,17,19,23,29,31,37,41,43,47,53,59,61,67,71,73,79,83,89,97};
+    mpz_init(a);
+    for (i = 0; i < 25 && res == -1; i++) {
+      mpz_set_ui(a, sprimes[i]);
+      if (mpz_jacobi(a, N) == -1)
+        res = 0;
+    }
+    /* (a,N) = -1 if res=0.  Do Proth primality test. */
+    if (res == 0) {
+      mpz_tdiv_q_2exp(k, v, 1);
+      mpz_powm(a, a, k, N);
+      if (mpz_cmp(a, v) == 0)
+        res = 2;
+    }
+    mpz_clear(a);
+  }
+  mpz_clear(k); mpz_clear(v);
+  if (res != -1 && get_verbose_level() > 1) printf("N shown %s with Proth\n", res ? "prime" : "composite");
+  fflush(stdout);
+  return res;
+}
 static int is_proth_form(mpz_t N)
 {
   mpz_t v, k;
@@ -540,7 +648,7 @@ int _GMP_is_lucas_pseudoprime(mpz_t n, int strength)
     mpz_tdiv_q_2exp(d, d, s);
   }
 
-  _GMP_lucas_seq(U, V, n, P, Q, d, Qk, t);
+  lucas_seq(U, V, n, P, Q, d, Qk, t);
   mpz_clear(d);
 
   rval = 0;
@@ -742,7 +850,7 @@ int is_frobenius_pseudoprime(mpz_t n, IV P, IV Q)
   mpz_init(t);
   if (P == 0 && Q == 0) {
     P = 1;  Q = 2;
-    while (k != -1) {
+    do {
       P += 2;
       if (P == 3) P = 5;  /* P=3,Q=2 -> D=9-8=1 => k=1, so skip */
       D = P*P-4*Q;
@@ -750,8 +858,9 @@ int is_frobenius_pseudoprime(mpz_t n, IV P, IV Q)
       if (mpz_cmp_ui(n, D >= 0 ? D : -D) <= 0) break;
       mpz_set_si(t, D);
       k = mpz_jacobi(t, n);
-      if (k != 1) break;
-    }
+    } while (k == 1);
+    if (k == 0 && (!mpz_cmp_ui(n,11) || !mpz_cmp_ui(n,13) || !mpz_cmp_ui(n,17)))
+      { mpz_clear(t); return 1; }
   } else {
     D = P*P-4*Q;
     mpz_set_si(t, D);
@@ -788,7 +897,7 @@ int is_frobenius_pseudoprime(mpz_t n, IV P, IV Q)
   if (k == 1) mpz_sub_ui(d, n, 1);
   else        mpz_add_ui(d, n, 1);
 
-  _GMP_lucas_seq(U, V, n, P, Q, d, Qk, t);
+  lucas_seq(U, V, n, P, Q, d, Qk, t);
   rval = ( mpz_sgn(U) == 0 && mpz_cmp(V, Vcomp) == 0 );
 
   mpz_clear(d); mpz_clear(Qk); mpz_clear(V); mpz_clear(U);
@@ -797,6 +906,101 @@ int is_frobenius_pseudoprime(mpz_t n, IV P, IV Q)
   return rval;
 }
 
+/* Use Crandall/Pomerance, steps from Loebenberger 2008 */
+int is_frobenius_cp_pseudoprime(mpz_t n, UV ntests)
+{
+  gmp_randstate_t* p_randstate = get_randstate();
+  mpz_t t, a, b, d, w1, wm, wm1, m;
+  UV tn;
+  int j;
+  int result = 1;
+
+  if (mpz_cmp_ui(n, 100) < 0)     /* tiny n */
+    return (_GMP_is_prob_prime(n) > 0);
+  if (mpz_even_p(n))
+    return 0;
+
+  mpz_init(t);  mpz_init(a);  mpz_init(b);  mpz_init(d);
+  mpz_init(w1);  mpz_init(wm);  mpz_init(wm1);  mpz_init(m);
+  for (tn = 0; tn < ntests; tn++) {
+    /* Step 1: choose a and b in 1..n-1 and d=a^2-4b not square and coprime */
+    do {
+      mpz_sub_ui(t, n, 1);
+      mpz_urandomm(a, *p_randstate, t);
+      mpz_add_ui(a, a, 1);
+      mpz_urandomm(b, *p_randstate, t);
+      mpz_add_ui(b, b, 1);
+      /* Check d and gcd */
+      mpz_mul(d, a, a);
+      mpz_mul_ui(t, b, 4);
+      mpz_sub(d, d, t);
+    } while (mpz_perfect_square_p(d));
+    mpz_mul(t, a, b);
+    mpz_mul(t, t, d);
+    mpz_gcd(t, t, n);
+    if (mpz_cmp_ui(t, 1) != 0 && mpz_cmp(t, n) != 0)
+      { result = 0; break; }
+    /* Step 2: W1 = a^2b^-1 - 2 mod n */
+    if (!mpz_invert(t, b, n))
+      { result = 0; break; }
+    mpz_mul(w1, a, a);
+    mpz_mul(w1, w1, t);
+    mpz_sub_ui(w1, w1, 2);
+    mpz_mod(w1, w1, n);
+    /* Step 3: m = (n-(d|n))/2 */
+    j = mpz_jacobi(d, n);
+    if (j == -1)     mpz_add_ui(m, n, 1);
+    else if (j == 0) mpz_set(m, n);
+    else if (j == 1) mpz_sub_ui(m, n, 1);
+    mpz_tdiv_q_2exp(m, m, 1);
+    /* Step 8 here:  B = b^(n-1)/2 mod n  (stored in d) */
+    mpz_sub_ui(t, n, 1);
+    mpz_tdiv_q_2exp(t, t, 1);
+    mpz_powm(d, b, t, n);
+    /* Quick Euler test */
+    mpz_sub_ui(t, n, 1);
+    if (mpz_cmp_ui(d, 1) != 0 && mpz_cmp(d, t) != 0)
+      { result = 0; break; }
+    /* Step 4: calculate Wm,Wm+1 */
+    mpz_set_ui(wm, 2);
+    mpz_set(wm1, w1);
+    {
+      UV b = mpz_sizeinbase(m, 2);
+      while (b-- > 0) {
+        if (mpz_tstbit(m, b)) {
+          mpz_mul(t, wm, wm1);
+          mpz_sub(wm, t, w1);
+          mpz_mul(t, wm1, wm1);
+          mpz_sub_ui(wm1, t, 2);
+        } else {
+          mpz_mul(t, wm, wm1);
+          mpz_sub(wm1, t, w1);
+          mpz_mul(t, wm, wm);
+          mpz_sub_ui(wm, t, 2);
+        }
+        mpz_mod(wm, wm, n);
+        mpz_mod(wm1, wm1, n);
+      }
+    }
+    /* Step 5-7: compare w1/wm */
+    mpz_mul(t, w1, wm);
+    mpz_mod(t, t, n);
+    mpz_mul_ui(wm1, wm1, 2);
+    mpz_mod(wm1, wm1, n);
+    if (mpz_cmp(t, wm1) != 0)
+      { result = 0; break; }
+    /* Step 8 was earlier */
+    /* Step 9: See if Bwm = 2 mod n */
+    mpz_mul(wm, wm, d);
+    mpz_mod(wm, wm, n);
+    if (mpz_cmp_ui(wm, 2) != 0)
+      { result = 0; break; }
+  }
+  mpz_clear(w1);  mpz_clear(wm);  mpz_clear(wm1);  mpz_clear(m);
+  mpz_clear(t);  mpz_clear(a);  mpz_clear(b);  mpz_clear(d);
+  return result;
+}
+
 /* New code based on draft paper */
 int _GMP_is_frobenius_underwood_pseudoprime(mpz_t n)
 {
@@ -869,6 +1073,74 @@ int _GMP_is_frobenius_underwood_pseudoprime(mpz_t n)
   return rval;
 }
 
+int _GMP_is_frobenius_khashin_pseudoprime(mpz_t n)
+{
+  mpz_t t, ta, tb, ra, rb, a, b, d;
+  unsigned long c = 1;
+  int k, rval = 0;
+
+  {
+    int cmpr = mpz_cmp_ui(n, 2);
+    if (cmpr == 0)     return 1;  /* 2 is prime */
+    if (cmpr < 0)      return 0;  /* below 2 is composite */
+    if (mpz_even_p(n)) return 0;  /* multiple of 2 is composite */
+  }
+  if (mpz_perfect_square_p(n)) return 0;
+
+  mpz_init(t);
+  do {
+    c += 2;
+    mpz_set_ui(t, c);
+    k = mpz_jacobi(t, n);
+  } while (k == 1);
+  if (k == 0) {
+    mpz_clear(t);
+    return 0;
+  }
+
+  mpz_init_set_ui(ra, 1);   mpz_init_set_ui(rb, 1);
+  mpz_init_set_ui(a, 1);    mpz_init_set_ui(b, 1);
+  mpz_init(ta);   mpz_init(tb);
+  mpz_init(d);
+  mpz_sub_ui(d, n, 1);
+
+  while (mpz_sgn(d)) {
+    if (mpz_odd_p(d)) {
+      mpz_mul(ta, ra, a);
+      mpz_mul(tb, rb, b);
+      mpz_add(t, ra, rb);
+      mpz_add(rb, a, b);
+      mpz_mul(rb, rb, t);
+      mpz_sub(rb, rb, ta);
+      mpz_sub(rb, rb, tb);
+      mpz_mod(rb, rb, n);
+      mpz_mul_ui(tb, tb, c);
+      mpz_add(ra, ta, tb);
+      mpz_mod(ra, ra, n);
+    }
+    mpz_tdiv_q_2exp(d, d, 1);
+    if (mpz_sgn(d)) {
+      mpz_mul(t, b, b);
+      mpz_mul_ui(t, t, c);
+      mpz_mul(b, b, a);
+      mpz_add(b, b, b);
+      mpz_mod(b, b, n);
+      mpz_mul(a, a, a);
+      mpz_add(a, a, t);
+      mpz_mod(a, a, n);
+    }
+  }
+  mpz_sub_ui(d, n, 1);
+  if ( (mpz_cmp_ui(ra,1) == 0) && (mpz_cmp(rb, d) == 0) )
+    rval = 1;
+
+  mpz_clear(ta); mpz_clear(tb);
+  mpz_clear(ra); mpz_clear(rb);
+  mpz_clear(a); mpz_clear(b);
+  mpz_clear(t);
+  return rval;
+}
+
 
 UV _GMP_trial_factor(mpz_t n, UV from_n, UV to_n)
 {
@@ -1161,6 +1433,10 @@ int _GMP_is_prime(mpz_t n)
   prob_prime = llr(n);
   if (prob_prime == 0 || prob_prime == 2) return prob_prime;
 
+  /* If the number is of form N=k*2^n+1 and we have a fast proof, do it. */
+  prob_prime = proth(n);
+  if (prob_prime == 0 || prob_prime == 2) return prob_prime;
+
   /* Start with BPSW */
   prob_prime = _GMP_BPSW(n);
   nbits = mpz_sizeinbase(n, 2);
@@ -1227,10 +1503,12 @@ int _GMP_is_provable_prime(mpz_t n, char** prooftext)
   int prob_prime = primality_pretest(n);
   if (prob_prime != 1)  return prob_prime;
 
-  /* Try LLR if they don't need a proof certificate. */
+  /* Try LLR and Proth if they don't need a proof certificate. */
   if (prooftext == 0) {
     prob_prime = llr(n);
     if (prob_prime == 0 || prob_prime == 2) return prob_prime;
+    prob_prime = proth(n);
+    if (prob_prime == 0 || prob_prime == 2) return prob_prime;
   }
 
   /* Start with BPSW */
@@ -1453,7 +1731,7 @@ int _GMP_is_aks_prime(mpz_t n)
 
 #elif AKS_VARIANT == AKS_VARIANT_BERNEXAMPLE
   {
-    double slim, x;
+    double slim, scmp, x;
     mpz_t t, t2;
     PRIME_ITERATOR(iter);
     mpz_init(t);  mpz_init(t2);
@@ -1461,25 +1739,21 @@ int _GMP_is_aks_prime(mpz_t n)
     s = 0;
     while (1) {
       UV q, tmp;
+      /* todo: Check r|n and r >= sqrt(n) */
       if (mpz_cmp_ui(n, r) <= 0) break;
       r = prime_iterator_next(&iter);
       q = largest_factor(r-1);
       mpz_set_ui(t, r);
       mpz_powm_ui(t, n, (r-1)/q, t);
-      /* gmp_printf("r %lu  q %lu  t %Zd\n", r, q, t); */
       if (mpz_cmp_ui(t, 1) <= 0) continue;
-      /* printf("trying r=%lu\n", r); */
 
-      s = 2;
-      slim = 2 * floor(sqrt(r)) * log(mpz_get_d(n));
-      while (1) {
+      slim = 40 * (r-1);
+      scmp = 2 * floor(sqrt(r)) * log(mpz_get_d(n))/log(2);
+      for (s = 2; s < slim; s++) {
         mpz_bin_uiui(t, q+s-1, s);
-        x = log(mpz_get_d(t)) / slim - 1.0;
-        if (x > 0) break;
-        s++;
-        if (s > 1000000) { s=0; break; }
+        if (log(mpz_get_d(t))/log(2) > scmp) break;
       }
-      if (s != 0) break;
+      if (s < slim) break;
     }
     mpz_clear(t);  mpz_clear(t2);
     prime_iterator_destroy(&iter);
@@ -1755,6 +2029,66 @@ void bernfrac(mpz_t num, mpz_t den, mpz_t zn)
   for (k = 1; k <= n; k++)  mpz_clear(T[k]);
   Safefree(T);
 }
+static void _harmonic(mpz_t a, mpz_t b, mpz_t t) {
+  mpz_sub(t, b, a);
+  if (mpz_cmp_ui(t, 1) == 0) {
+    mpz_set(b, a);
+    mpz_set_ui(a, 1);
+  } else {
+    mpz_t q, r;
+    mpz_add(t, a, b);
+    mpz_tdiv_q_2exp(t, t, 1);
+    mpz_init_set(q, t); mpz_init_set(r, t);
+    _harmonic(a, q, t);
+    _harmonic(r, b, t);
+    mpz_mul(a, a, b);
+    mpz_mul(t, q, r);
+    mpz_add(a, a, t);
+    mpz_mul(b, b, q);
+    mpz_clear(q); mpz_clear(r);
+  }
+}
+
+void harmfrac(mpz_t num, mpz_t den, mpz_t zn)
+{
+  mpz_t t;
+  mpz_init(t);
+  mpz_add_ui(den, zn, 1);
+  mpz_set_ui(num, 1);
+  _harmonic(num, den, t);
+  mpz_gcd(t, num, den);
+  mpz_divexact(num, num, t);
+  mpz_divexact(den, den, t);
+  mpz_clear(t);
+}
+
+char* harmreal(mpz_t zn, unsigned long prec) {
+  char* out;
+  mpz_t num, den;
+  mpf_t fnum, fden, res;
+  unsigned long numsize, densize;
+
+  mpz_init(num); mpz_init(den);
+  harmfrac(num, den, zn);
+
+  numsize = mpz_sizeinbase(num, 10);
+  densize = mpz_sizeinbase(den, 10);
+
+  mpf_init2(fnum, 1 + mpz_sizeinbase(num,2));
+  mpf_init2(fden, 1 + mpz_sizeinbase(den,2));
+  mpf_set_z(fnum, num);  mpf_set_z(fden, den);
+  mpz_clear(den); mpz_clear(num);
+
+  mpf_init2(res, (mp_bitcnt_t) (8+prec*3.4) );
+  mpf_div(res, fnum, fden);
+  mpf_clear(fnum);  mpf_clear(fden);
+
+  New(0, out, (10+numsize-densize)+prec, char);
+  gmp_sprintf(out, "%.*Ff", (int)(prec), res);
+  mpf_clear(res);
+
+  return out;
+}
 
 void stirling(mpz_t r, unsigned long n, unsigned long m, UV type)
 {
@@ -2594,24 +2928,28 @@ int _GMP_squfof_factor(mpz_t n, mpz_t f, UV rounds)
 /* See if n is a perfect power */
 UV power_factor(mpz_t n, mpz_t f)
 {
-  if (mpz_cmp_ui(n, 1) <= 0) return 0;
-  if (mpz_perfect_power_p(n)) {
-    UV k;
-    mpz_set_ui(f, 1);
-    for (k = 2; mpz_sgn(f); k++) {
-      if (mpz_root(f, n, k)) {
-        if (mpz_perfect_power_p(f)) {  /* If root is a power, recurse */
-          mpz_t nf;
-          mpz_init_set(nf, f);
-          k *= power_factor(nf, f);
-          mpz_clear(nf);
-        }
-        return k;
+  UV k = 1, b = 2;
+  if (mpz_cmp_ui(n, 1) > 0 && mpz_perfect_power_p(n)) {
+    mpz_t nf, tf;
+    PRIME_ITERATOR(iter);
+
+    mpz_init_set(nf, n);
+    mpz_init(tf);
+    while (1) {
+      UV ok = k;
+      while (mpz_root(tf, nf, b)) {
+        mpz_set(f, tf);
+        mpz_set(nf, tf);
+        k *= b;
       }
+      if (ok != k && !mpz_perfect_power_p(nf)) break;
+      if (mpz_cmp_ui(tf, 1) <= 0) break; /* Exit if we can't find the power */
+      b = prime_iterator_next(&iter);
     }
-    /* GMP says it's a perfect power, but we couldn't find an integer root? */
+    mpz_clear(tf);  mpz_clear(nf);
+    prime_iterator_destroy(&iter);
   }
-  return 0;
+  return (k == 1) ? 0 : k;
 }
 
 /* a=0, return power.  a>1, return bool if an a-th power */
