@@ -5,7 +5,7 @@ use Carp qw/croak confess carp/;
 
 BEGIN {
   $Math::Prime::Util::GMP::AUTHORITY = 'cpan:DANAJ';
-  $Math::Prime::Util::GMP::VERSION = '0.29';
+  $Math::Prime::Util::GMP::VERSION = '0.30';
 }
 
 # parent is cleaner, and in the Perl 5.10.1 / 5.12.0 core, but not earlier.
@@ -29,8 +29,10 @@ our @EXPORT_OK = qw(
                      is_perrin_pseudoprime
                      is_frobenius_pseudoprime
                      is_frobenius_underwood_pseudoprime
+                     is_frobenius_khashin_pseudoprime
                      is_mersenne_prime
                      is_llr_prime
+                     is_proth_prime
                      miller_rabin_random
                      lucas_sequence  lucasu  lucasv
                      primes
@@ -53,7 +55,7 @@ our @EXPORT_OK = qw(
                      pn_primorial
                      factorial
                      consecutive_integer_lcm
-                     partitions bernfrac stirling
+                     partitions bernfrac harmfrac harmreal stirling
                      gcd lcm kronecker valuation invmod binomial gcdext
                      vecsum vecprod
                      exp_mangoldt
@@ -168,7 +170,7 @@ __END__
 
 =encoding utf8
 
-=for stopwords Möbius Deléglise Bézout gcdext vecsum vecprod moebius totient liouville znorder znprimroot bernfrac stirling lucasu lucasv OpenPFGW gmpy2
+=for stopwords Möbius Deléglise Bézout gcdext vecsum vecprod moebius totient liouville znorder znprimroot bernfrac harmfrac harmreal stirling lucasu lucasv OpenPFGW gmpy2 nonresidue
 
 =head1 NAME
 
@@ -177,7 +179,7 @@ Math::Prime::Util::GMP - Utilities related to prime numbers and factoring, using
 
 =head1 VERSION
 
-Version 0.29
+Version 0.30
 
 
 =head1 SYNOPSIS
@@ -548,9 +550,19 @@ C<(x+2)^(n+1) = 2a + 5 mod (x^2-ax+1,n)>.  This combines a Fermat and Lucas
 test at a computational cost of about 2.5x a strong pseudoprime test.  This
 makes it similar to, but faster than, a Frobenius test.
 
-There are no known pseudoprimes to this test.  This test also has no overlap
-with the BPSW test, making it a very effective method for adding additional
-certainty.
+This test is deterministic (no randomness is used).  There are no known
+pseudoprimes to this test.  This test also has no overlap with the BPSW
+test, making it a very effective method for adding additional certainty.
+
+=head2 is_frobenius_khashin_pseudoprime
+
+Takes a positive number as input, and returns 1 if the input passes the
+Frobenius test of Sergey Khashin.  This ensures C<n> is not a perfect square,
+selects the parameter C<c> as the smallest odd prime such that C<(c|n)=-1>,
+then verifies that C<(1+D)^n = (1-D) mod n> where C<D = sqrt(c) mod n>.
+
+This test is deterministic (no randomness is used).
+There are no known pseudoprimes to this test.
 
 =head2 is_bpsw_prime
 
@@ -593,16 +605,32 @@ faster than any known general-form primality proof methods.
 
 Takes a positive number C<n> as input and returns one of: 0 (definitely
 composite), 2 (definitely prime), or -1 (test does not indicate anything).
-This implements a partial version of the Lucas-Lehmer-Riesel test for
+This implements the Lucas-Lehmer-Riesel test for
 fast deterministic primality testing on numbers of the form C<k * 2^n - 1>.
 If C<k = 1> then this is a Mersenne number and the Lucas-Lehmer test is used.
 If the number is not of this form, or if C<k E<lt>= 2^n>, then C<-1> will
-be returned as the test does not apply.  In this implementation, some values
-of C<k> are not solved, so they will also return C<-1>.  Otherwise, the LLR
+be returned as the test does not apply.
+Otherwise, the LLR
 test is performed.  While not as fast as the Lucas-Lehmer test for Mersenne
 numbers, it is almost as fast as a single strong pseudoprime test (i.e.
 Miller-Rabin test) while giving a certain answer.
 
+=head2 is_proth_prime
+
+Takes a positive number C<n> as input and returns one of: 0 (definitely
+composite), 2 (definitely prime), or -1 (test does not indicate anything).
+This applies Proth's theorem for fast Las Vegas
+primality testing on numbers of the form C<k * 2^n + 1>.
+If the number is not of this form, or if C<k E<lt>= 2^n>, then C<-1> will
+be returned as the test does not apply.
+Otherwise, a search is performed to find a quadratic nonresidue modulo C<n>.
+If none can be found after a brief search, C<-1> is returned as no conclusion
+can be reached.  Otherwise, Proth's theorem is checked which conclusively
+indicates primality.
+While not as fast as the Lucas-Lehmer test for Mersenne
+numbers, it is almost as fast as a single strong pseudoprime test (i.e.
+Miller-Rabin test) while giving a certain answer.
+
 =head2 is_nminus1_prime
 
   say "$n is definitely prime" if is_nminus1_prime($n);
@@ -826,10 +854,24 @@ C<n E<lt> 0, k > 0> are undefined.
 =head2 bernfrac
 
 Returns the Bernoulli number C<B_n> for an integer argument C<n>, as a
-rational number represented by two L<Math::BigInt> objects.  B_1 = 1/2.
+rational number.  Two values are returned, the numerator and denominator.
+B_1 = 1/2.
 This corresponds to Pari's C<bernfrac(n)> and Mathematica's C<BernoulliB>
 functions.
 
+=head2 harmfrac
+
+Returns the Harmonic number C<H_n> for an integer argument C<n>, as a
+rational number.  Two values are returned, the numerator and denominator.
+numbers are the sum of reciprocals of the first C<n> natural numbers:
+C<1 + 1/2 + 1/3 + ... + 1/n>.
+
+=head2 harmreal
+
+Returns the Harmonic number C<H_n> for an integer argument C<n>, as a
+string floating point.  An optional second argument indicates the number
+of digits to be preserved past the decimal place, with a default of 40.
+
 =head2 stirling
 
   say "s(14,2) = ", stirling(14, 2);
@@ -1372,7 +1414,7 @@ ECM implementation, as well as the papers by Brent and Montgomery.
 
 =head1 COPYRIGHT
 
-Copyright 2011-2014 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
+Copyright 2011-2015 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
 
 This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
